<?php
/**
 * Dofus_DB_Interactor.class.php
 * 
 * File to hold the database interactor class and all error classes
 * 
 * @author Jim Wordelman <j.wordelman@quad341.com>
 * @package Dofus
 * @subpackage Database
 */

require_once "Dofus_DB.abstractclass.php";
/**
 * Exception class for errors from executing queries
 *
 */
class Dofus_DB_Query_Execution_Exception extends Exception {}
/**
 * Exception class for an invalid login
 *
 */
class Dofus_DB_Login_Exception extends Exception {}
/**
 * Class to interact with the database
 *
 */
class Dofus_DB_Interactor extends Dofus_DB {
	/**
	 * The database link identifier
	 *
	 * @var int
	 */
	private $dbh = false;
	/**
	 * Constructor to initiate the connection to the database
	 *
	 * @param string[optional] $dbhost The host for the database (usually localhost) default defined in Dofus_DB class
	 * @param string[optional] $dbuser The username for the database; default set in Dofus_DB class
	 * @param string[optional] $dbpass The password for the database; default set in Dofus_DB class
	 * @param string[optional] $dbname The name of the database to chose; default set in Dofus_DB class
	 */
	public function __construct($dbhost=Dofus_DB::DB_HOST,$dbuser=Dofus_DB::DB_USER,$dbpass=Dofus_DB::DB_PASS,$dbname=Dofus_DB::DB_NAME){
		$dbh = mysql_connect($dbhost,$dbuser,$dbpass);
		if(!$dbh) throw new Dofus_DB_Invalid_Credentials_Exception('Invalid Login');
		mysql_select_db($dbname,$dbh);
		$this->dbh = $dbh;
	}
	/**
	 * Adds a category
	 *
	 * @param string $cat_name The name for the category
	 */
	public function addCategory($cat_name){
		$query = "INSERT INTO categories (category) VALUES ('$cat_name')";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Updates the supplied category
	 *
	 * @param int $id The id of the category to update
	 * @param string $cat_name The new name for the category
	 */
	public function updateCategory($id,$cat_name){
		$query = "UPDATE categories SET category='$cat_name' WHERE cat_id='$id'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Selects the category specified by id
	 *
	 * @param int $id The id of the category
	 * @return resource The mysql result
	 */
	public function selectCategory($id){
		$query = "SELECT category FROM categories WHERE cat_id='$id'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
		return $result;
	}
	/**
	 * Deletes the supplied category
	 *
	 * @param int $id The id of the category to delete
	 */
	public function deleteCategory($id){
		$query = "DELETE FROM categories WHERE cat_id='$id'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Adds an item
	 *
	 * @param string $item The name of the item
	 * @param int $category_id The id of the category to associate with the item
	 */
	public function addItem($item,$category_id){
		$query = "INSERT INTO items (item,category) VALUES ('$item','$category_id')";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Updates the supplied item with the information
	 *
	 * @param int $id The id of the item to update
	 * @param string $item The name of the item
	 * @param int $cat_id The id of the category
	 */
	public function editItem($id,$item,$cat_id){
		$query = "UPDATE items SET item='$item',category='$cat_id' WHERE item_id='$id'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Selects an item by the supplied id
	 *
	 * @param int $id The id of the item
	 * @return resource The mysql result
	 */
	public function selectItem($id){
		$query = "SELECT item,category FROM items WHERE item_id='$id'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
		return $result;
	}
	/**
	 * Deletes the supplied item
	 *
	 * @param int $id The id of the item to delete
	 */
	public function deleteItem($id){
		$query = "DELETE FROM items WHERE item_id='$id'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Adds a price entry
	 *
	 * @param int $price The price of the item
	 * @param int $item_id The id of the item
	 * @param int $location_id The id of the location
	 * @param int $quantity The quantity of the item that was available
	 * @param int $submitter_id The user_id of the person who submitted the price
	 * @param string $comments Any additional comments about the item
	 */
	public function addPrice($price,$item_id,$location_id,$quantity,$submitter_id,$comments){
		$query = "INSERT INTO prices (price,item_id,location_id,quantity,timestamp,submitter_id,comments) VALUES ('$price','$item_id','$location_id','$quantity',GETDATE(),'$submitter_id','$comments')";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Deletes the price entry
	 *
	 * @param int $id The id of the price
	 */
	public function deletePrice($id){
		$query = "DELETE FROM prices WHERE price_id='$id'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Selects an item and the average price associated with it from all time
	 *
	 * @param int $item_id The id of the item to select
	 * @return resource The mysql result
	 */
	public function selectItemPrice($item_id){
		$query = "(SELECT items.item,items.category AS category_id,categories.category FROM items INNER JOIN categories ON items.category=categories.id WHERE items.id='$id') UNION (SELECT AVE(price) AS ave_price FROM prices WHERE item_id='$id')";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
		return $result;
	}
	/**
	 * Selects an item and prices from the past month
	 *
	 * @param int $item_id The id of the item to select
	 * @return resource The mysql result
	 */
	public function selectItemMonth($item_id){
		$query = "(SELECT items.item,items.category AS category_id,categories.category FROM items INNER JOIN categories ON items.category=categories.id WHERE items.id='$id') UNION (SELECT price,AVE(price) AS ave_price FROM prices WHERE item_id='$id' AND timestamp=?????)";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
		return $result;
	}
	/**
	 * Attempts to select the login information from the supplied user with the password
	 *
	 * @param string $username The user's username
	 * @param string $passhash The hashed password of the user
	 * @return resource The mysql result of login information
	 */
	public function login($username,$passhash){
		$query = "SELECT user_id,username,email,permission FROM users WHERE username='$username' AND passhash='$passhash' AND verified='1'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
		if(mysql_num_rows($result)) throw new Dofus_DB_Login_Exception($username);
		return $result;
	}
	/**
	 * Adds a user (non-verified)
	 *
	 * @param string $username The username of the user to add
	 * @param string $email The email address of the user
	 * @param string $passhash The password hash
	 * @param int $permission The permission level of the user
	 */
	public function addUser($username,$email,$passhash,$permission){
		$query = "INSERT INTO users (username,email,passhash,permission,verified) VALUES ('$username','$email','$passhash','$permission','0')";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Sets the user to verified status
	 *
	 * @param int $id The id of the user to verify
	 */
	public function verifyUser($id){
		$query = "UPDATE users SET verified='1' WHERE id='$id'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
	/**
	 * Updates the supplied user's information
	 *
	 * @param int $id The id of the user to update
	 * @param string $email The email account of the user
	 * @param string $passhash The hash of the user's password
	 * @param int $permission The permission level for the user
	 */
	public function updateUser($id,$email,$passhash,$permission){
		$query = "UPDATE users SET email='$email',passhash='$passhash',permission='$permission' WHERE id='$id'";
		$result = mysql_query($query,$this->dbh);
		if(!$result) throw new Dofus_DB_Query_Execution_Exception($query);
	}
}
?>